import axios from 'axios'
import rgbHex from 'rgb-hex'
import color from 'css-color-function'
import useThemeStore from '@/stores/theme'
import formula from '@/global/constants/formula.json'

// 1. [rgb-hex](https://www.npmjs.com/package/rgb-hex)：转换RGB(A)颜色为十六进制
// 2. [css-color-function](https://www.npmjs.com/package/css-color-function)：在CSS中提出的颜色函数的解析器和转换器

/**
 * 把生成的样式表写入到 style 中
 * @param {*} elNewStyle  element-plus 的新样式
 */
export const writeNewStyle = (elNewStyle: string) => {
  const themeStore = useThemeStore()
  const styleEl = document.createElement('style')
  styleEl.textContent = elNewStyle
  document.head.append(styleEl)
  // 先移除之前添加的, 再重新指向新插入的 style 元素
  if (themeStore.styleEl instanceof HTMLStyleElement) {
    themeStore.styleEl.remove()
  }
  themeStore.setStyleEl(styleEl)
}

/**
 * 根据主题色, 生成新的样式表
 * @param primaryColor 主题色
 */
export const generateNewStyle = async (primaryColor: string): Promise<any> => {
  // 1. 根据主色生成色值表
  const colors = generateColors(primaryColor)
  if (!colors) return
  // 2. 获取当前 element-plus 的默认样式表, 并且把需要进行替换的色值打上标记
  let cssText: string = await getOriginalStyle()
  // 3. 遍历生成的色值表, 在 默认样式表 中进行全局替换
  Object.keys(colors).forEach((tag) => {
    // $n 表示第 n 个被捕获括号(分组)匹配的子字符串
    cssText = cssText.replace(
      new RegExp('(:|\\s+)' + tag, 'g'),
      '$1' + colors[tag]
    )
  })

  return cssText
}

/**
 * 获取基于主题色生成的系列色值表
 * @param primary 主题色
 * @returns 基于主题色生成的系列色
 */
export const generateColors = (primary: string) => {
  if (!primary) return

  const colors: any = {
    primary
  }

  // 基于主题色 primary, 通过增加 白色（tint） 和 黑色（shade） 的占比来生成系列色
  Object.keys(formula).forEach((key: any) => {
    const value = formula[key].replace(/primary/g, primary)
    // 将 value 转换成一个十六进制颜色(不包含 #)
    colors[key] = '#' + rgbHex(color.convert(value))
  })

  return colors
}

/**
 * 获取当前 element-plus 的默认样式表
 */
export const getOriginalStyle = async () => {
  // https://unpkg.com/element-plus@2.4.3/dist/index.css
  // const elementPlusVersion = (await import('element-plus/package.json')).version
  // css 的 cdn 链接直接从官网获取即可
  // https://element-plus.org/zh-CN/guide/installation.html#jsdelivr
  // const url = `https://unpkg.com/element-plus@${elementPlusVersion}/dist/index.css`
  const url = `https://cdn.jsdelivr.net/npm/element-plus/dist/index.css`

  try {
    const { data } = await axios.get(url)
    return getStyleTemplate(data)
  } catch (error: any) {
    return Promise.reject(new Error(error))
  }
}

/**
 * 给需要进行替换的色值打上标记
 */
const getStyleTemplate = (data: any) => {
  interface IColorMap {
    [key: string]: string
  }
  // element-plus 默认色值
  const colorMap: IColorMap = {
    // 左边是右边的 element-plus 原色值
    '#3a8ee6': 'shade-1',
    '#409eff': 'primary',
    '#53a8ff': 'light-1',
    '#66b1ff': 'light-2',
    '#79bbff': 'light-3',
    '#8cc5ff': 'light-4',
    '#a0cfff': 'light-5',
    '#b3d8ff': 'light-6',
    '#c6e2ff': 'light-7',
    '#d9ecff': 'light-8',
    '#ecf5ff': 'light-9'
  }

  Object.keys(colorMap).forEach((key) => {
    const tag = colorMap[key]

    // 将 data 中所有的 key 替换为 colorMap[key]
    data = data.replace(new RegExp(key, 'ig'), tag)
  })

  return data
}
